package ru.indvdum.mywork.vaadin;

import static ru.indvdum.mywork.MyWork.EM;

import java.sql.Time;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import ru.indvdum.mywork.openjpa.model.Day;

import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.Property.ValueChangeListener;
import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.PopupDateField;
import com.vaadin.ui.TextField;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;

public class EditDayDialog extends Window implements ValueChangeListener, ClickListener {
	
	private MyWorkApplication myApp = null;
	private EditState state = null;
	private Day day = null;
	private PopupDateField dayStartTime = null;
	private PopupDateField dayEndTime = null;
	private PopupDateField dayDate = null;
	private TextField dayHours = null;

	public EditDayDialog(MyWorkApplication myApp, EditState state, Day day) throws Exception {
		super();
		this.myApp = myApp;
		this.state = state;
		switch(state){
		case CREATE:
			setCaption("Creating new day");
			if(day != null)
				throw new Exception("Illegal argument: day must be null in creating state.");
			day = new Day();
			Calendar calDay = Calendar.getInstance();
			calDay.set(Calendar.HOUR_OF_DAY, 0);
			calDay.set(Calendar.MINUTE, 0);
			calDay.set(Calendar.SECOND, 0);
			calDay.set(Calendar.MILLISECOND, 0);
			day.setDay(calDay.getTime());
			Calendar calStart = Calendar.getInstance();
			calStart.set(Calendar.HOUR_OF_DAY, 9);
			calStart.set(Calendar.MINUTE, 0);
			calStart.set(Calendar.SECOND, 0);
			calStart.set(Calendar.MILLISECOND, 0);
			day.setStartWork(new Time(calStart.getTime().getTime()));
			Calendar calEnd = Calendar.getInstance();
			calEnd.set(Calendar.HOUR_OF_DAY, 18);
			calEnd.set(Calendar.MINUTE, 0);
			calEnd.set(Calendar.SECOND, 0);
			calEnd.set(Calendar.MILLISECOND, 0);
			day.setEndWork(new Time(calEnd.getTime().getTime()));
			day.setHours(8f);
			this.day = day;
			break;
		case EDIT:
			setCaption("Editing day");
			if(day == null)
				throw new Exception("Illegal argument: day must be not null in editing state.");
			this.day = day;
			break;
		}
		init();
	}
	
	private void init(){
		setModal(true);
		setWidth("480px");
		setHeight("300px");
		VerticalLayout vl = new VerticalLayout();
		vl.setSizeFull();
		vl.setSpacing(true);
		vl.setMargin(true);
		setContent(vl);
		
		HorizontalLayout hlDay = new HorizontalLayout();
		hlDay.setWidth("100%");
		Label lblDay = new Label("Day:");
		lblDay.setWidth("100%");
		hlDay.addComponent(lblDay);
		hlDay.setExpandRatio(lblDay, 1f);
		dayDate = new PopupDateField();
		dayDate.setWidth("100%");
		dayDate.setInputPrompt("Enter date");
		dayDate.setResolution(PopupDateField.RESOLUTION_DAY);
		dayDate.setImmediate(true);
		dayDate.setValue(day.getDay());
		dayDate.addListener(this);
		dayDate.setDateFormat("dd.MM.yyyy");
		hlDay.addComponent(dayDate);
		hlDay.setExpandRatio(dayDate, 5f);
		vl.addComponent(hlDay);
		
		HorizontalLayout hlStart = new HorizontalLayout();
		hlStart.setWidth("100%");
		Label lblStart = new Label("Start time:");
		lblStart.setWidth("100%");
		hlStart.addComponent(lblStart);
		hlStart.setExpandRatio(lblStart, 1f);
		dayStartTime = new PopupDateField();
		dayStartTime.setWidth("100%");
		dayStartTime.setInputPrompt("Enter time");
		dayStartTime.setResolution(PopupDateField.RESOLUTION_MIN);
		dayStartTime.setImmediate(true);
		dayStartTime.setValue(day.getStartWork());
		dayStartTime.setDateFormat("HH:mm");
		hlStart.addComponent(dayStartTime);
		hlStart.setExpandRatio(dayStartTime, 5f);
		vl.addComponent(hlStart);
		
		HorizontalLayout hlEnd = new HorizontalLayout();
		hlEnd.setWidth("100%");
		Label lblEnd = new Label("End time:");
		lblEnd.setWidth("100%");
		hlEnd.addComponent(lblEnd);
		hlEnd.setExpandRatio(lblEnd, 1f);
		dayEndTime = new PopupDateField();
		dayEndTime.setWidth("100%");
		dayEndTime.setInputPrompt("Enter time");
		dayEndTime.setResolution(PopupDateField.RESOLUTION_MIN);
		dayEndTime.setImmediate(true);
		dayEndTime.setValue(day.getEndWork());
		dayEndTime.setDateFormat("HH:mm");
		hlEnd.addComponent(dayEndTime);
		hlEnd.setExpandRatio(dayEndTime, 5f);
		vl.addComponent(hlEnd);
		
		HorizontalLayout hlHours = new HorizontalLayout();
		hlHours.setWidth("100%");
		Label lblHours = new Label("Work hours count:");
		lblHours.setWidth("100%");
		hlHours.addComponent(lblHours);
		hlHours.setExpandRatio(lblHours, 1f);
		dayHours = new TextField();
		dayHours.setWidth("100%");
		dayHours.setValue(day.getHours());
		dayHours.setImmediate(true);
		hlHours.addComponent(dayHours);
		hlHours.setExpandRatio(dayHours, 5f);
		vl.addComponent(hlHours);
		
		HorizontalLayout hlOk = new HorizontalLayout();
		hlOk.setSizeFull();
		Button okButton = new Button();
		switch(state){
		case CREATE:
			okButton.setCaption("Create");
			break;
		case EDIT:
			okButton.setCaption("Save");
			break;
		}
		okButton.addListener((ClickListener)this);
		hlOk.addComponent(okButton);
		hlOk.setComponentAlignment(okButton, Alignment.BOTTOM_RIGHT);
		vl.addComponent(hlOk);
		vl.setExpandRatio(hlOk, 1f);
	}

	@Override
	public void valueChange(ValueChangeEvent event) {
		Object value = event.getProperty().getValue();
		Object startValue = dayStartTime.getValue();
		Object endValue = dayEndTime.getValue();
		if (value == null || !(value instanceof Date) || !(startValue instanceof Date) || !(endValue instanceof Date)) {
			getWindow().showNotification("Invalid date entered");
		} else {
			Date date = (Date)value;
			Date start = (Date)startValue;
			Date end = (Date)endValue;
			Calendar calStart = Calendar.getInstance();
			calStart.setTime(start);
			Calendar calEnd = Calendar.getInstance();
			calEnd.setTime(end);
			Calendar cal = Calendar.getInstance();
			cal.setTime(date);
			calStart.set(Calendar.YEAR, cal.get(Calendar.YEAR));
			calStart.set(Calendar.MONTH, cal.get(Calendar.MONTH));
			calStart.set(Calendar.DAY_OF_MONTH, cal.get(Calendar.DAY_OF_MONTH));
			calStart.set(Calendar.SECOND, cal.get(Calendar.SECOND));
			calStart.set(Calendar.MILLISECOND, cal.get(Calendar.MILLISECOND));
			calEnd.set(Calendar.YEAR, cal.get(Calendar.YEAR));
			calEnd.set(Calendar.MONTH, cal.get(Calendar.MONTH));
			calEnd.set(Calendar.DAY_OF_MONTH, cal.get(Calendar.DAY_OF_MONTH));
			calEnd.set(Calendar.SECOND, cal.get(Calendar.SECOND));
			calEnd.set(Calendar.MILLISECOND, cal.get(Calendar.MILLISECOND));
			dayStartTime.setValue(calStart.getTime());
			dayEndTime.setValue(calEnd.getTime());
		}
	}

	@Override
	public void buttonClick(ClickEvent event) {
		Object value = dayDate.getValue();
		Object startValue = dayStartTime.getValue();
		Object endValue = dayEndTime.getValue();
		Object hoursValue = dayHours.getValue();
		if (
				value == null 
				|| !(value instanceof Date) 
				|| !(startValue instanceof Date) 
				|| !(endValue instanceof Date)
				|| !(hoursValue.toString().matches("^[0-9]+[\\.]{0,1}[0-9]*"))
				) {
			getWindow().showNotification("Invalid information entered");
		} else {
			Date date = (Date)value;
			Date start = (Date)startValue;
			Date end = (Date)endValue;
			Float hours = Float.parseFloat(hoursValue.toString());
			day.setDay(date);
			day.setStartWork(new Time(start.getTime()));
			day.setEndWork(new Time(end.getTime()));
			day.setHours(hours);
			try{
				EM.getTransaction().begin();
				EM.persist(day);
				EM.getTransaction().commit();
				EM.clear();
			} finally {
				if(EM.getTransaction().isActive())
					EM.getTransaction().rollback();
			}
			getParent().removeWindow(this);
			myApp.getDayTable().requestRepaint();
		}
	}

}
